// The pin used to connect the sensor
#define PIN_DATA_IN 4

// On a Atmega 128 only has 1K ram, so use 200 readings max
// On a Atmega 328 only has 2K ram, so use 420 readings max
// A value too high here will use all the memory and crash the Arduino
#define MAX_READINGS 300

// Timeout to send the values anyway even if we didn't get the desired number of readings
// This value is in MICROseconds, not MILIseconds
#define TIMEOUT_VALUE 4800000

int number_of_readings;
byte input_active_high;

// Used to set a timeout
unsigned long start_time;
unsigned long end_time;
unsigned long current_time;

// Stores all the values read here
unsigned long readings[MAX_READINGS];
int current_reading = 0;

int readSequences()
{
//    Serial.println(number_of_readigs);
    // Registers all the state changes
    for ( current_reading = 0;  current_reading <number_of_readings; current_reading++)
    {
        // Waits untill it reads a HIGH or LOW value, depending on the input type.
        if (input_active_high)
        {
            while (!digitalRead(PIN_DATA_IN))
            {
                if (micros() > end_time) return current_reading;
            }
        }
        else
        {
            while (digitalRead(PIN_DATA_IN))
            {
                if (micros() > end_time) return current_reading;
            }
        }
        // When the first value comes, saves that time
        readings[current_reading] = micros();
        digitalWrite(13, HIGH);

        // Wait untill the input changes back again
        if (input_active_high)
        {
            while (digitalRead(PIN_DATA_IN))
            {
                if (micros() > end_time) return current_reading;
            }
        }
        else
        {
            while (!digitalRead(PIN_DATA_IN))
            {
                if (micros() > end_time) return current_reading;
            }
        }
        current_reading++;
        readings[current_reading] = micros();
    }
    return current_reading;
}

// Sends the data to the serial port
void sendSequences()
{
    // After reading all the values, sends then to the computer
    for ( current_reading = 0;  current_reading < number_of_readings; current_reading++)
    {    
        Serial.write((readings[current_reading] & 0xFF ));
        Serial.write((readings[current_reading] >> 8) & 0xFF );
        Serial.write((readings[current_reading] >> 16) & 0xFF );
        Serial.write((readings[current_reading] >> 24) & 0xFF );
    }
}

// Waits for a reading request from SequenceDecoder
void waitForRequest()
{
    // Starts by waiting for a readings request
    while (Serial.read() != 255);
    while (Serial.read() != 255);  
    while (Serial.read() != 255);
    // Now wait for the number of values to read
    while (Serial.available() <2);
    byte size_lo = Serial.read();
    byte size_hi = Serial.read();
    number_of_readings = ( size_hi << 8 | size_lo );
    // And finally wait for the input type
    while (Serial.available() <1);
    input_active_high = Serial.read();

    // Lights up the LED when the readings start
    digitalWrite(13, HIGH);
    start_time = micros();
    end_time = start_time+TIMEOUT_VALUE;
    number_of_readings = readSequences();
    sendSequences();
    Serial.flush();
    // Turns off the LED when it finishes reading
    digitalWrite(13, LOW);
}

void setup() 
{
    Serial.begin(115200);    
    pinMode(13, OUTPUT);    
    digitalWrite(13, LOW);
}

void loop() 
{
    waitForRequest();
}
